__author__ = 'sandeep'
import multiprocessing

from consumer import Consumer

import logging

LOGGER = logging.getLogger(__name__)


class HedwigWorker(multiprocessing.Process):
    def __init__(self, settings, *args, **kwargs):
        self.hedwig_consumer = Consumer(settings)
        super(HedwigWorker, self).__init__(*args, **kwargs)

    def run(self):
        LOGGER.info("Hedwig consumer: starting")
        self.hedwig_consumer.consume()
        LOGGER.info("hedwig consumer: stopped")

    def shutdown(self):
        LOGGER.info("Hedwig consumer: shutting down")
        self.hedwig_consumer.shutdown()
        LOGGER.info("Hedwig consumer: shutdown complete")



__author__ = 'sandeep'
import multiprocessing

import logging

LOGGER = logging.getLogger(__name__)


class DjangoHedwigWorker(multiprocessing.Process):
    def __init__(self, *args, **kwargs):
        self.hedwig_consumer = None
        super(DjangoHedwigWorker, self).__init__(*args, **kwargs)

    def run(self):
        from hedwig.core.consumer import Consumer
        import django
        django.setup()
        from .settings import hedwig_settings
        self.hedwig_consumer = Consumer(hedwig_settings)
        LOGGER.info("Django Hedwig consumer: starting")
        self.hedwig_consumer.consume()
        LOGGER.info("Django hedwig consumer: stopped")

    def shutdown(self):
        LOGGER.info("Django Hedwig consumer: shutting down")
        if self.hedwig_consumer is not None:
            self.hedwig_consumer.shutdown()
        LOGGER.info("Django Hedwig consumer: shutdown complete")



__author__ = 'sandeep'
import signal
import time
import sys

import multiprocessing
import inspect
import traceback
import logging


LOGGER = logging.getLogger(__name__)


class ServiceManager(object):
    def __init__(self, worker_cls, num_workers=2):
        if not (inspect.isclass(worker_cls) and issubclass(worker_cls, multiprocessing.Process)):
            raise ValueError("Worker class must extend multiprocessing.Process")
        self.workers = []
        self.num_workers = num_workers
        self.worker_cls = worker_cls
        self.running = False
        signal.signal(signal.SIGINT, self.stop)
        signal.signal(signal.SIGTERM, self.stop)

    def start(self, *args, **kwargs):
        self.running = True
        LOGGER.info("Starting: Hedwig Worker Service...")
        try:
            while self.running:
                if len(self.workers) < self.num_workers:
                    worker = self.worker_cls(*args, **kwargs)
                    worker.start()
                    self.workers.append(worker)
                else:
                    for worker in self.workers:
                        if not worker.is_alive():
                            worker.shutdown()
                            self.workers.remove(worker)
                            worker.terminate()
                    time.sleep(5)

        except KeyboardInterrupt:
            self.stop()
        except Exception as e:
            LOGGER.exception(str(e))
            traceback.print_exc(file=sys.stdout)
            self.stop()

    def stop(self, signum=None, frame=None):
        self.running = False
        LOGGER.info("Stopping: Hedwig Worker Service...")
        for worker in self.workers:
            LOGGER.info("Stopping Worker: " + worker.name)
            worker.shutdown()
            worker.terminate()
        sys.exit(0)

    def is_running(self):
        return self.running



__author__ = 'piyush'
print """Hedwig

Jab yahan se 50 kos door koi developer SOA me communication ke liye rota hai,
To uska TL kehta chup ho ja, hum Hedwig laga denge.

Hedwig is a highly extensible cross language set of libraries to implement reliable communication between services.
To achieve this, it uses the emitter-consumer model on top of AMQP.

Principles of Hedwig

Keep it simple and reliable.
Do not repeat yourself.
Trust the developers to do a good job.
There should be one good default to plug and play.
Explicit is better than implicit. (H/t Zen of Python, Tim Peters)
Safe implicit is better than nothing.
Allow for powerful configuration and extension.
Keep reusable implementation and business use cases separate.
Contribute the reusable implementation back to the community.
Reliability and usability beat everything.
"""



from rest_framework.serializers import ModelSerializer
from rest_framework.renderers import JSONRenderer
from hedwig.django.settings import project_name
from .settings import hedwig_rest_framework_settings
from hedwig.django.emitter import hedwig_emitter


class HedwigModelSerializer(ModelSerializer):
    def save(self, **kwargs):
        action = 'created'
        if self.instance is not None:
            action = 'updated'
        super(HedwigModelSerializer, self).save(**kwargs)
        emit_hedwig_serializer_data(action, self.__module__.split('.')[0], self.__class__.__name__, self)
        return self.instance


def emit_hedwig_serializer_data(action, module_name, serializer_name, serializer):
    if not bool(hedwig_rest_framework_settings.SERIALIZER_SIGNALS):
        return

    routing_key = '.'.join([project_name, module_name, 'serializer', serializer_name, action,
                            str(serializer.instance.pk)])
    json_renderer = JSONRenderer()
    hedwig_emitter.emit(routing_key, json_renderer.render(serializer.data))
